from sqlalchemy import create_engine
from sqlalchemy import Column, Table
from sqlalchemy import Integer, String, ForeignKey
from sqlalchemy.orm import relationship
from sqlalchemy.orm import sessionmaker
from sqlalchemy.engine import Engine
from sqlalchemy.ext.declarative import declarative_base


MODEL_BASE = declarative_base()


class BaseInterface(object):
    def __init__(self, engine: Engine):
        self.engine = engine
        self.session_maker = sessionmaker(bind=self.engine)
        self.session = self.session_maker()

    @classmethod
    def from_db_uri(cls, uri: str):
        engine = create_engine(uri)
        return cls(engine)

    def create(self, *args, **kwargs):
        raise NotImplementedError

    def read_one(self, *args, **kwargs):
        raise NotImplementedError

    def read_all(self, *args, **kwargs):
        raise NotImplementedError

    def update(self, *args, **kwargs):
        raise NotImplementedError

    def delete(self, *args, **kwargs):
        raise NotImplementedError


def initiate_database(uri):
    engine = create_engine(uri)
    MODEL_BASE.metadata.create_all(engine)
    return engine



from typing import Optional, List

from .base import BaseInterface
from .models import Cocktail, Ingredient
from .serializers import CocktailSchema, IngredientSchema


class CocktailInterface(BaseInterface):
    def create(self, name:str, ingredients:List[int]):
        new_item = Cocktail(name=name)

        for ingredient_id in ingredients:
            ingredient_object = IngredientInterface.read_one(ingredient_id)
            Cocktail.ingredients.append(ingredient_object)

        self.session.add(new_item)
        self.session.commit()
        return new_item


    def read_one(self, id:int):
        query = self.session.query(Cocktail).filter_by(Cocktail.id==id)
        found = query.one()
        return found

    def read_all(self):
        query = self.session.query(Cocktail)
        found = query.all()
        return found


class IngredientInterface(BaseInterface):
    def create(self, name: str):
        new_item = Ingredient(name=name)
        self.session.add(new_item)
        self.session.commit()
        return new_item

    def read_one(self, id: int):
        query = self.session.query(Ingredient).filter_by(Ingredient.id == id)
        found = query.one()
        return found

    def read_all(self):
        query = self.session.query(Ingredient)
        found = query.all()
        return found



from .base import MODEL_BASE
from .base import Column, Table, String, Integer, ForeignKey
from .base import relationship


ingredient_cocktail_association = Table(
    'ingredient_cocktail_association',
    MODEL_BASE.metadata,
    Column('ingredient_id', Integer, ForeignKey('ingredient.id'), primary_key=True),
    Column('cocktail_id', Integer, ForeignKey('cocktail.id'), primary_key=True),
)


class Ingredient(MODEL_BASE):
    __tablename__ = 'ingredient'
    id = Column(Integer, primary_key=True)
    name = Column(String(80), unique=True, nullable=False)


class Cocktail(MODEL_BASE):
    __tablename__ = 'cocktail'
    id = Column(Integer, primary_key=True)
    name = Column(String(80), unique=True, nullable=False)
    ingredients = relationship(
        'Ingredient',
        secondary='ingredient_cocktail_association',
    )


from marshmallow import Schema, fields


class IngredientSchema(Schema):
    id = fields.Integer()
    name = fields.String()


class CocktailSchema(Schema):
    id = fields.Integer()
    name = fields.String()
    ingredients = fields.Nested(IngredientSchema, many=True)



import os
from setuptools import setup


setup(
    name = 'cocktail_model_lib',
    version = '0.0.1',
    author='Brandon Purvis',
    author_email='bpurvis.career@gmail.com',
    description='Library of tools for managing a database on cocktails.',
    license='MIT',
    keywords='cocktail sqlalchemy database',
    url='https://github.com/brandonPurvis/CocktailModelLib',
    packages=['cocktail_model_lib'],
    package_data={
        '': ['*.txt', '*.rst', '*.md'],
    },
    install_requires=[
        'SQLAlchemy',
        'marshmallow',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
    ]
)
